<?php

/**
 * @file
 * FanCourier class.
 */

/**
 * FanCourier class. Handle api calls to fancourier server.
 */
class FanCourier {

  // @codingStandardsIgnoreStart
  // Url settings.
  private $api_url = 'http://www.selfawb.ro';
  private $request_type = array(
    // A pair of request type and the endpoint that has to be added to the URL.
    'services' => 'order.php',
    'estimation' => 'tarif.php',
  );
  // User login settings.
  private $username = NULL;
  private $user_pass = NULL;
  private $client_id = NULL;
  // Service list response, raw and processed.
  private $services_raw = array();
  private $services = array();
  // Errors.
  private $shipping_options = array(
    'localitate_dest' => '',
    'judet_dest' => '',
    'plicuri' => '0',
    'colete' => '0',
    'greutate' => '0',
    'lungime' => '',
    'latime' => '',
    'inaltime' => '',
    'plata_la' => 'destinatar',
    'plata_ramburs' => '1',
  );
  // @codingStandardsIgnoreEnd
  // Shipping.
  private $errors = array();

  /**
   * Constructor.
   *
   * @param array $data
   *   Login data.
   *
   * @return type object \FanCourier
   */
  public function __construct($data) {
    // Validate that required params are set.
    foreach (array('username', 'user_pass', 'client_id') as $var) {
      if (empty($data[$var])) {
        $this->setError("Empty {$var} in " . __CLASS__ . ', line ' . __LINE__, 'general');

        return;
      }

      $this->$var = $data[$var];
    }
    // Calculate the service list.
    $this->getServices();
  }

  /**
   * Get the service list.
   * 
   * @param bool $reset
   *   Optional, whether to reset the service list, otherwise use cached data.
   *
   * @return array
   *   The service list array.
   */
  public function getServices($reset = FALSE) {
    if (!empty($this->errors)) {
      return $this;
    }

    if (!empty($this->services) && !$reset) {
      return $this->services;
    }

    $request_type = 'services';
    $services = $this->request($request_type);

    if (!$services) {
      $this->setError('No services returned by request in ' . __CLASS__ . ', line ' . __LINE__, 'services');
      return $this;
    }

    $this->services_raw = str_getcsv($services, "\n");
    foreach ($this->services_raw as $service_info) {
      $service_details = explode(',', $service_info);
      $this->services[$service_details[0]] = $service_details[0];
    }

    return $this->services;
  }

  /**
   * Default values for shipping.
   *
   * @return array
   *   Default shipping settings.
   */
  public function getShippingDefaults() {
    return array(
      'plata_la' => 'destinatar',
      'colete' => '1',
      'greutate' => '1',
      'plata_ramburs' => '1',
    );
  }

  /**
   * Estimate the shipping.
   *
   * @param array $options
   *   Shipping options.
   * @param string $service
   *   Shipping service, default to 'Standard'.
   *
   * @return mixed
   *   The response from an estimate shipping request.
   */
  public function estimateShipping($options, $service = 'Standard') {
    if (!empty($this->errors)) {
      return $this;
    }

    $default_options = $this->getShippingDefaults();
    $default_options['serviciu'] = $service;
    $this->shipping_options = array_merge($default_options, $options);
    $estimation = $this->request('estimation', $this->shipping_options);

    if (!$estimation) {
      $this->setError('No estimation returned by request in ' . __CLASS__ . ', line ' . __LINE__, 'shipping');
      return $this;
    }

    if (!is_numeric($estimation)) {
      $this->setError('Error in ' . __CLASS__ . ', line ' . __LINE__ . ' with message: ' . $estimation, 'shipping');
      return $this;
    }

    return $estimation;
  }

  /**
   * Generic request method for both "services" and "estimation".
   *
   * @param string $request_type
   *   A valid request string as pressent in $request_type.
   * @param array $options
   *   An optional array of post fields.
   * 
   * @return mixed
   *   The response of a request to the API.
   */
  private function request($request_type = NULL, $options = NULL) {
    // Validate the request type.
    if (!$request_type || !isset($this->request_type[$request_type])) {
      $this->setError('Request type param is missing.');
    }

    // Validate URL?
    $url = $this->api_url . '/' . $this->request_type[$request_type];

    // Add options, if any provided.
    if (is_array($options)) {
      $post_fields = $options;
    }
    // Make sure we have all the required params and add them to post fields.
    foreach (array('username', 'user_pass', 'client_id') as $var) {
      if (empty($this->$var)) {
        $this->setError("Invalid {$var} in " . __CLASS__ . ', line ' . __LINE__, 'request');

        return $this;
      }
      $post_fields[$var] = $this->$var;
    }

    $post_fields['return'] = $request_type;

    $c = curl_init($url);
    curl_setopt($c, CURLOPT_POST, TRUE);
    curl_setopt($c, CURLOPT_POSTFIELDS, http_build_query($post_fields));
    curl_setopt($c, CURLOPT_RETURNTRANSFER, TRUE);

    $result = FALSE;

    try {
      $result = curl_exec($c);
    }
    catch (Exception $e) {
      $this->setError('Curl error: "' . $e->getMessage() . '" in ' . __CLASS__ . ', line ' . __LINE__, 'request');
      curl_close($c);
      return $this;
    }

    curl_close($c);
    return $result;
  }

  /**
   * Generic getter.
   */
  public function set($var = NULL, $value = NULL) {
    if (!$var || $value === NULL) {
      return $this;
    }

    if (property_exists($this, $var)) {
      $this->{$var} = $value;
    }

    return $this;
  }

  /**
   * Generic setter.
   */
  public function get($var = NULL) {
    if (!$var || !isset($this->$var)) {
      return $this;
    }

    return $this->{$var};
  }

  /**
   * Error setter.
   */
  private function setError($message, $scope = 'general', $level = FALSE) {
    $this->errors[$scope] = $message;

    $levels = array(
      'notice' => E_USER_NOTICE,
      'warning' => E_USER_WARNING,
      'error' => E_USER_ERROR,
    );

    // Used for debug.
    if ($level) {
      if (!isset($levels[$level])) {
        $level = 'notice';
      }
      trigger_error($message, $levels[$level]);
    }
  }

  /**
   * Error getter.
   * 
   * @return array
   *   Error array list.
   */
  public function getErrors() {
    return $this->errors;
  }

  /**
   * Check if there are any errors.
   * 
   * @return bool
   *   Wheter there are any errors.
   */
  public function hasErrors() {
    return empty($this->errors) ? FALSE : TRUE;
  }

}
